unit MainFormU;

{$DEFINE USE_CUSTOM_MSG_BOX}

interface

uses
  System.SysUtils, System.Types, System.UITypes, System.Classes, System.Variants,
  FMX.Types, FMX.Controls, FMX.Forms, FMX.Graphics, FMX.Dialogs,
  Androidapi.JNI.GraphicsContentViewText, FMX.ListBox, FMX.Edit, FMX.StdCtrls,
  FMX.Layouts;

type
  TMainForm = class(TForm)
    Layout: TLayout;
    TelNoButton: TButton;
    BatteryUsageButton: TButton;
    WebURLButton: TButton;
    MapURLButton: TButton;
    StreetViewURLButton: TButton;
    ContactsButton: TButton;
    SMSButton: TButton;
    EmailButton: TButton;
    BarcodeScannerButton: TButton;
    VertScrollBox: TVertScrollBox;
    procedure FormCreate(Sender: TObject);
    procedure TelNoButtonClick(Sender: TObject);
    procedure BatteryUsageButtonClick(Sender: TObject);
    procedure BarcodeScannerButtonClick(Sender: TObject);
    procedure WebURLButtonClick(Sender: TObject);
    procedure MapURLButtonClick(Sender: TObject);
    procedure StreetViewURLButtonClick(Sender: TObject);
    procedure ContactsButtonClick(Sender: TObject);
    procedure SMSButtonClick(Sender: TObject);
    procedure EmailButtonClick(Sender: TObject);
  private
    { Private declarations }
    const ScanRequestCode = 0;
    procedure RegisterDelphiNativeMethods;
    function OnActivityResult(RequestCode, ResultCode: Integer; Data: JIntent): Boolean;
  public
    { Public declarations }
  end;

var
  MainForm: TMainForm;

implementation

uses
  FMX.Helpers.Android,
  Androidapi.NativeActivity,
  Androidapi.JNI,
  Androidapi.JNIBridge,
  Androidapi.JNI.JavaTypes,
  Androidapi.JNI.App,
  Androidapi.JNI.Embarcadero,
  Androidapi.JNI.Toast,
  LaunchActivities;

{$R *.fmx}

{$REGION 'JNI setup code and callback for onActivityResult() native'}
//This is called from the Java activity's onActivityResult() method
function OnActivityResultNative(PEnv: PJNIEnv; This: JNIObject;
  RequestCode, ResultCode: Integer; dataIntent: JNIObject): Boolean; cdecl;
var
  SyncResult: Boolean;
begin
  Log.d('Queuing native routine to run synchronized');
  TThread.Queue(nil,
    procedure
    var
      Data: JIntent;
    begin
      Log.d('+ThreadSwitcher');
      Log.d('Thread: Main: %.8x, Current: %.8x, Java:%.8d (%2:.8x)', [MainThreadID, TThread.CurrentThread.ThreadID,
        TJThread.JavaClass.CurrentThread.getId]);
      Data := nil;
      if Assigned(DataIntent) then
        Data := TJIntent.Wrap(DataIntent);
      SyncResult := MainForm.OnActivityResult(requestCode, resultCode, Data);
      Log.d('-ThreadSwitcher');
    end);
  Result := SyncResult;
end;

procedure TMainForm.RegisterDelphiNativeMethods;
var
  PEnv: PJNIEnv;
  ActivityClass: JNIClass;
  NativeMethod: JNINativeMethod;
begin
  PEnv := TJNIResolver.GetJNIEnv;
  NativeMethod.Name := 'onActivityResultNative';
  NativeMethod.Signature := '(IILandroid/content/Intent;)Z';
  NativeMethod.FnPtr := @OnActivityResultNative;

  ActivityClass := PEnv^.GetObjectClass(
    PEnv, PANativeActivity(System.DelphiActivity).clazz);

  PEnv^.RegisterNatives(PEnv, ActivityClass, @NativeMethod, 1);

  PEnv^.DeleteLocalRef(PEnv, ActivityClass);
end;
{$ENDREGION}

{$REGION 'Message box wrappers'}
type
  JBlongNativeActivity = interface;

  JBlongNativeActivityClass = interface(JFMXNativeActivityClass)
  ['{4D6F9029-4169-45B0-9EE7-F1E6EFAF3D01}']
    {Methods}
    function init: JBlongNativeActivity; cdecl;
  end;

  [JavaSignature('com/blong/test/NativeActivitySubclass')]
  JBlongNativeActivity = interface(JFMXNativeActivity)
  ['{181C0AB5-FF5A-48A7-B051-547FE6F17778}']
    {Methods}
    procedure showDialog(title, msg: JString); cdecl;
  end;
  TJBlongNativeActivity = class(TJavaGenericImport<JBlongNativeActivityClass, JBlongNativeActivity>) end;

//There is a bug logged internally regarding the FMX message box routines
//(ShowMessage & MessageDlg) hanging when called from mobile OS callbacks
//Use of custom message boxes (say implemented in Java) avoids this, but
//in this example they are non-blocking
procedure NativeShowMsg(const Title, Msg: String);
var
  Activity: JBlongNativeActivity;
begin
  CallInUIThread(
    procedure
    begin
      Activity := TJBlongNativeActivity.Wrap(PANativeActivity(System.DelphiActivity)^.clazz);
      Activity.showDialog(StringToJString(Title), StringToJString(Msg));
    end);
end;

procedure ShowMessage(const Msg: String);
begin
  NativeShowMsg(Application.Title, Msg)
end;

function MessageDlg(const Msg: string; DlgType: TMsgDlgType;
  Buttons: TMsgDlgButtons; HelpCtx: Integer): Integer;
var
  Title: string;
begin
  case DlgType of
    TMsgDlgType.mtInformation: Title := 'Information';
    TMsgDlgType.mtWarning: Title := 'Warning';
    TMsgDlgType.mtError: Title := 'Error';
  else
    Title := Application.Title
  end;
  NativeShowMsg(Title, Msg);
  Result := 0; //Non-blocking routine can't pass back a result
end;
{$ENDREGION}

{ TMainForm }

procedure TMainForm.FormCreate(Sender: TObject);
begin
  Log.d('TMainForm.FormCreate');
  RegisterDelphiNativeMethods
end;

procedure TMainForm.BarcodeScannerButtonClick(Sender: TObject);
begin
  LaunchQRScanner(ScanRequestCode);
end;

procedure TMainForm.BatteryUsageButtonClick(Sender: TObject);
begin
  ShowBatteryUsage
end;

procedure TMainForm.ContactsButtonClick(Sender: TObject);
begin
  LaunchUrl('content://contacts/people')
end;

procedure TMainForm.EmailButtonClick(Sender: TObject);
begin
  CreateEmail(['foo@bar.com'], 'Important Info', 'O HAI!')
end;

procedure TMainForm.MapURLButtonClick(Sender: TObject);
begin
  LaunchUrl('geo:37.422,-122.084?z=17')
end;

procedure TMainForm.SMSButtonClick(Sender: TObject);
begin
  CreateSMS('123456789', 'O HAI!')
end;

procedure TMainForm.StreetViewURLButtonClick(Sender: TObject);
begin
  LaunchUrl('google.streetview:cbll=37.422044,-122.083849&cbp=12,7.59,,0,-34.22&mz=21')
end;

procedure TMainForm.TelNoButtonClick(Sender: TObject);
begin
  LaunchUrl('tel:123456789')
end;

procedure TMainForm.WebURLButtonClick(Sender: TObject);
begin
  LaunchUrl('http://www.embarcadero.com/delphi')
end;

function TMainForm.OnActivityResult(RequestCode, ResultCode: Integer;
  Data: JIntent): Boolean;
var
  ScanContent,
  ScanFormat: string;
begin
  Result := False;

  //For more info see https://github.com/zxing/zxing/wiki/Scanning-Via-Intent
  if RequestCode = ScanRequestCode then
  begin
    if ResultCode = TJActivity.JavaClass.RESULT_OK then
    begin
      if Assigned(Data) then
      begin
        ScanContent := JStringToString(Data.getStringExtra(StringToJString('SCAN_RESULT')));
        ScanFormat := JStringToString(Data.getStringExtra(StringToJString('SCAN_RESULT_FORMAT')));
        //Toast(Format('Found %s format barcode:'#10'%s', [ScanFormat, ScanContent]), LongToast);
        ShowMessage(Format('Content = %s'#10#10'Format = %s', [ScanContent, ScanFormat]))
      end;
    end
    else if ResultCode = TJActivity.JavaClass.RESULT_CANCELED then
    begin
      //Toast('You cancelled the scan', ShortToast);
      ShowMessage('You cancelled the scan....')
    end;
    Result := True;
  end;
end;

end.
